// --------------------------------------------------------------------
// Copyright (c) 2007 by Terasic Technologies Inc. 
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// --------------------------------------------------------------------
//           
//                     Terasic Technologies Inc
//                     356 Fu-Shin E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include "alt_video_display.h"
#include "app_selector.h"
#include "my_ui.h"
#include "simple_graphics.h"
#include "fonts.h"
#include "sys/alt_alarm.h"
#include "sys/alt_irq.h"  // interrupt
#include <unistd.h>
#include "gimp_bmp.h"

#include "terasic_includes.h"
#include "I2C.h"
#include "ADXL345.h"
#include "math.h"
#include "light_sensor.h"


#define COLOR_R 0xFF0000
#define COLOR_G 0x00FF00
#define COLOR_B 0x0000FF
#define COLOR_WHITE 0xFFFFFF
#define COLOR_BLACK 0x000000
#define COLOR_GRAY  0x7F7F7F

#define V_BAR_X  30
#define V_BAR_Y  (80-30)

#define H_BAR_X  140
#define H_BAR_Y  (428-30)

#define CENTER_BALL_X  180
#define CENTER_BALL_Y  (100-30)

#define R 70

#define ENABLE_BUTTON 0

buttons_struct command_buttons;

void draw_border(int color, alt_video_display* display){
    int x,y;
    alt_u8 *pDes = display->buffer_ptrs[display->buffer_being_written]->buffer;
    for(y=0;y<display->height;y++){
        for(x=0;x<display->width;x++){
            if (x<=1 || x >= display->width-2 || y <= 1 || y >= display->height-2){
                *pDes++ = color & 0xFF; // blue
                *pDes++ = (color >> 8) & 0xFF; // green
                *pDes++ = (color >> 16) & 0xFF; // red
                pDes++;                 // reserved
            }else{
                pDes += 4;
            }
        }
    }
    
}

int my_ui( alt_video_display* display){
    int time_start, border_index=0, color;
    char szText[128];
    alt_16 x, y , z;
    
    float x_angle, y_angle;
    float x_offset_angle = 0.0, y_offset_angle = 0.0;
    
    Bar h_Bar = {0,0, 600, 325, 438-30 };
    Bar v_Bar = {0, 0, 600, 40,  265-30};
    
    Ball center_Ball = {0,0, 125, 289, 210-30};
    
    //temp display
    void *temp_Buffer;
    temp_Buffer =(void*)malloc(display->bytes_per_frame);
    
    //printf("x= %d , y = %d \n", h_Bar.get_X(0,h_Bar.origin_x), h_Bar.get_Y(0,h_Bar.origin_y));
    
    printf("===== welcome to G_sensor demo program =====\n");
    usleep(200*1000);
    
    
    extern struct gimp_image_struct ball_water;
    extern struct gimp_image_struct v_bar_water;
    extern struct gimp_image_struct h_bar_water;
    
    extern struct gimp_image_struct h_Bar_Line;
    extern struct gimp_image_struct v_Bar_Line;
    
    extern struct gimp_image_struct ring;
    
    extern struct gimp_image_struct dot;
    
    int nX= 0, nY = 0;
    int center_x, center_y ;
    
    InitBackGround(display, temp_Buffer);
    
    //light sensor
    alt_u16 light0 = 0, light1 = 0;
    bool bPass = FALSE;
    IOWR_ALTERA_AVALON_PIO_DIRECTION(LIGHT_ADDR_SEL_BASE,1);
    IOWR_ALTERA_AVALON_PIO_DATA(LIGHT_ADDR_SEL_BASE, 0);
    bPass = Light_PowerSwitch(TRUE);
    if(bPass)
    {
        printf("light sensor power up ok...\n");
    }
    else
    {
        printf("light sensor power up fail...\n");
        return -1;
    }
  
    ////////////////////////Button END//////////////////////////////////
    
      while(1)
      {
        
        if(!Gsensor_getXYZ(&x,&y,&z))
        {
           printf("Gsensor_getXYZ failed!\n");
           return 0;
        }
        

        x_angle = x_offset_angle - atan(x*1.0/z)*180.0/M_PI;
        y_angle = atan(y*1.0/z)*180.0/M_PI - y_offset_angle;
        
        if(y_angle < - 10 && y_angle > -80)
        {
            //lcd direction
            IOWR(LCD_LEFT_RIGHT_BASE, 0, 0);
            IOWR(LCD_UP_DOWN_BASE, 0, 1);
        }
        else if(y_angle < 80 && y_angle > 10)
        {
            //lcd direction
            IOWR(LCD_LEFT_RIGHT_BASE, 0, 1);
            IOWR(LCD_UP_DOWN_BASE, 0, 0);
        }
        
        nX = (int)(x_angle * 140 /90);
        nY = (int)(y_angle * 140 /90);
     
        memcpy((void*)(display->buffer_ptrs[display->buffer_being_written]->buffer),temp_Buffer ,display->bytes_per_frame);
             
        //draw water
        if(nX >= 0)
        {
            center_x = nX > R? R:nX;
        }
        else
        {
            center_x = nX < R*(-1) ? R*(-1) : nX;
        }
        
        if(nY >= 0)
        {
            center_y = nY > R ? R : nY;
        }
        else
        {
            center_y = nY < R*(-1) ? R*(-1) : nY; 
        }
       
        //draw ball water
        DrawTransparentImage(display, &ball_water, center_x+center_Ball.origin_x, center_Ball.origin_y-center_y, 0x0073c301);
       
         //draw ring
        DrawTransparentImage(display, &ring, center_Ball.origin_x-13, center_Ball.origin_y-13, 0x0073c301);
        
        //DrawImage(display, &ball_water, center_x+h_Bar.origin_x, center_Ball.origin_y-center_y);
        DrawImage(display, &v_bar_water, v_Bar.origin_x, v_Bar.origin_y-nY);
        DrawImage(display, &h_bar_water, nX+h_Bar.origin_x, h_Bar.origin_y);
        
        //draw Line
        DrawImage(display, &h_Bar_Line, h_Bar.origin_x-5, h_Bar.origin_y-2);
        DrawImage(display, &h_Bar_Line, h_Bar.origin_x+41, h_Bar.origin_y-2);
        
        DrawImage(display, &v_Bar_Line, v_Bar.origin_x-2, v_Bar.origin_y-6);
        DrawImage(display, &v_Bar_Line, v_Bar.origin_x-2, v_Bar.origin_y+40);
        
        //display x angle
        sprintf(szText, "%d", (int)(x_angle));
        vid_print_string_alpha(480, 380-30, COLOR_BLACK,COLOR_WHITE, tahomabold_32, display, szText);
        int length = vid_string_pixel_length_alpha(tahomabold_32, szText);
        DrawImage(display, &dot, 480+length, 390-30);
        
        //display y angle
        sprintf(szText, "%d", (int)(y_angle));
        vid_print_string_alpha(90, 100-30, COLOR_BLACK,COLOR_WHITE, tahomabold_32, display, szText);
        int length1 = vid_string_pixel_length_alpha(tahomabold_32, szText);
        DrawImage(display, &dot, 90+length1, 110-30);
        
       ///////////////////////////////////////light sensor////////////////////////////////////////////////////////////
        char szText1[128] = {0};
        sprintf(szText, "%d", 0);
        sprintf(szText1, "%d", 0);
        bPass = Get_light(&light0,&light1);
        if(bPass)
        {
            sprintf(szText,  "%d", light0);
            sprintf(szText1, "%d", light1);
        }
        vid_print_string_alpha(630, 55, COLOR_WHITE,-1, tahomabold_32, display, szText);
        vid_print_string_alpha(630, 115, COLOR_WHITE,-1, tahomabold_32, display, szText1);
        
        alt_video_display_register_written_buffer( display );
     }  // for i
}

void DrawImage( alt_video_display* display, struct gimp_image_struct* Terasic_image, int horiz, int vert )
{

  bitmap_struct* image;
  char* image_dest;

  image = malloc(sizeof(bitmap_struct));
  
  if( image != NULL )
  {
    // Load the CIII logo
    load_gimp_bmp( Terasic_image, image, 32);

      image_dest = (char*)(( display->buffer_ptrs[display->buffer_being_written]->buffer ) + (vert * (display->width * 4)) + horiz * 4); 
                              
      // Copy the image to the current display buffer
      AsVidCopyImageToBuffer( image_dest,
                              image->data, 
                              display->width, 
                              image->biWidth, 
                              image->biHeight );     

    free_gimp_bmp_data( image );
    free( image );
  }
}

int AsVidCopyImageToBuffer( char* dest, char* src, 
                            int dest_width, 
                            int src_width, int src_height )
{
  int y;
  
  //Copy one line at a time from top to bottom
  for ( y = 0; y < src_height; y++ )
  {
    memcpy( dest, src, ( src_width * 4 ));
    src += ( src_width * 4 );
    dest += ( dest_width * 4 );
  }
  
  return( 0 );
} 

////draw array 
void draw_array(unsigned int *buffer, int x_size, int y_size,
  alt_video_display* display, int x_offset, int y_offset,
  unsigned int transparent_color)
{
  unsigned int *current_buffer;
  int addr = ( (int)(display->buffer_ptrs[display->buffer_being_written]->buffer) );
  current_buffer = (unsigned int *)addr;
  int x_b,y_b;
  unsigned int color;

  transparent_color = 0xFF000000 | transparent_color;

  for (x_b = 0; x_b<x_size; x_b++) {
    for (y_b = 0; y_b<y_size; y_b++) {
      color = buffer[y_b*x_size+x_b];
      if (color != transparent_color) {
        current_buffer[(y_b+y_offset)*display->width+(x_b+x_offset)] = color;
      }
    }
  }
}

void InitBackGround(alt_video_display* display, void *temp_Buffer)
{
    extern struct gimp_image_struct H_Bar;
    extern struct gimp_image_struct V_Bar;
    extern struct gimp_image_struct ball;   
    extern struct gimp_image_struct bright;
    
    alt_video_display_clear_screen( display, COLOR_WHITE );
        
        //draw v_Bar , h_Bar, Ball
        DrawImage(display, &V_Bar, V_BAR_X,V_BAR_Y);
        DrawImage(display, &ball, CENTER_BALL_X, CENTER_BALL_Y);
        DrawImage(display, &H_Bar, H_BAR_X,H_BAR_Y);
        DrawImage(display, &bright,  570, 30);
              
        memcpy(temp_Buffer, (void*)(display->buffer_ptrs[display->buffer_being_written]->buffer),display->bytes_per_frame);
}

void DrawTransparentImage( alt_video_display* display, struct gimp_image_struct* Terasic_image, int horiz, int vert,unsigned int transparent_color )
{

  bitmap_struct* image;
  char* image_dest;

  image = malloc(sizeof(bitmap_struct));
  
  if( image != NULL )
  {
    // Load the CIII logo
    load_gimp_bmp( Terasic_image, image, 32);

      image_dest = (char*)(( display->buffer_ptrs[display->buffer_being_written]->buffer ) + (vert * (display->width * 4)) + horiz * 4); 
                              
      // Copy the image to the current display buffer
//      AsVidCopyImageToBuffer( image_dest,
//                              image->data, 
//                              display->width, 
//                              image->biWidth, 
//                              image->biHeight );   
  int y;
  //Copy one line at a time from top to bottom
//  for ( y = 0; y < image->biHeight; y++ )
//  {
//    memcpy( dest, src, ( image->biWidth * 4 ));
//    src += ( image->biWidth * 4 );
//    dest += ( display->width * 4 );
//  }  

    draw_array((unsigned int*)image->data,image->biWidth,image->biHeight,display,horiz,vert,transparent_color);

    free_gimp_bmp_data( image );
    free( image );
  }
}

int DRAW_CopyRGBImageToBuffer( char* dest, char* src, 
                            int dest_width, 
                            int src_width, int src_height )
{
  int x, y;
  
  
  //Copy one line at a time from top to bottom
  char *pSrcBase, *pDesBase;
  pSrcBase = src;
  pDesBase = dest;
  for ( y = 0; y < src_height; y++ )
  {
        char *pDes = pDesBase;
        char *pSrc = pSrcBase;
        for(x = 0; x< src_width; x++){
            *pDes++ = *pSrc++;
            *pDes++ = *pSrc++;
            *pDes++ = *pSrc++;
            *pDes++ = 0;  // alpha
        }
        
        pDesBase += dest_width * 4;
        pSrcBase += src_width * 3;
  }
  
  return( 0 );
}

bool Get_light(alt_u16 *light0, alt_u16 *light1)
{
    bool bPass = FALSE;
    alt_u8 device_Id = 0;

    alt_u16 ChannelData0;
    alt_u16 ChannelData1;
   
    //get light sensor Id
    Light_GetID(&device_Id);
    if(device_Id == 0)
    {
        printf("read id fail \n");
        return FALSE;
    }
    
    //get adc channel 0 data
    bPass = Light_Get_ADCData0(&ChannelData0);
    if(!bPass)
    {
        printf("Get ADC Channle 0 data fail \n");
        return FALSE;
    }
    *light0 = ChannelData0;
   
    //get adc channel 1 data
    bPass = Light_Get_ADCData1(&ChannelData1);
    if(!bPass)
    {
        printf("Get ADC Channle 1 data fail \n");
        return FALSE;
    }
    *light1 = ChannelData1;
    return TRUE;
}

