/*****************************************************************************
 *  File: app_selector.c
 * 
 *  This file contains functions for performing general tasks associated
 *  with the application selector.
 * 
 ****************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <sys/alt_alarm.h>
#include <string.h>
#include "sys/alt_cache.h"
#include "system.h"
#include "alt_types.h"
#include "io.h"
#include "app_selector.h"
#include "app_selector_gui.h"
#include "altera_avalon_pio_regs.h"
#include "alt_video_display.h"
#include "sd_controller.h"
#include "LCD.h"

#include "sys/alt_flash.h"
#include "sys/alt_flash_dev.h"
#include "altera_avalon_epcs_flash_controller.h"



/*****************************************************************************
 *  Function: AsFindAppsOnSD
 *
 *  Purpose: Looks through the directory of the SD Card where the loadable
 *           application are, and fills out a list of all the valid-looking
 *           applications it finds.
 * 
 *  Possible enhancements: Remove the error message displaying from here,
 *                         and just pass back an error code.
 * 
 *  Returns: 0     - Success
 *           non-0 - Failure
 ****************************************************************************/
int AsFindAppsOnSD( app_list_struct* sd_app_list )
{    
    int volumes_mounted;
     int nTryTime = 0;
    //Init sd card
    SD_CONTROLLER_INSTANCE( SD_CARD_CONTROLLER, sd_card_controller );
        
    for(nTryTime = 0; nTryTime <5 ; nTryTime ++)
    {
        SD_CONTROLLER_INIT( SD_CARD_CONTROLLER, sd_card_controller );
        ////set spi clock divider coeff.
        sd_set_clock_to_max( 80000000 );
        usleep (1000);
        ////mount fat file system.
        volumes_mounted = sd_fat_mount_all();
    
        if(volumes_mounted > 0)
        {
            break;
        }
    }
    if(nTryTime >= 5)
        return 1;
    
    char buf[800] = {0};
    sd_list("/Application_Selector/",buf);
    
    sd_app_list->num_apps = 0;
    
    int i=0;
    int start = -1;
    int end   = -1;
    
    char name[80] = {0};
    for(i=0;i<800;i++)
    {
        if(buf[i] !='\0' && start == -1) //start Dir
        {
            start = i;
            end = -1;
        }
        else if(buf[i] == '\0' && end == -1)//end Dir
        {
            end = i;
            memcpy(name,buf+start,end-start+1);
            
            if(name[0] != '.') //
            {
                strcpy( sd_app_list->apps[sd_app_list->num_apps].app_name, name );
                strcpy( sd_app_list->apps[sd_app_list->num_apps].sw_filename, "");
                strcpy( sd_app_list->apps[sd_app_list->num_apps].hw_filename, "");
                //strcpy(app_list[iIndex].Dir , name);
               
                //search hardware file and software file
                char tmpBuf[100] = {0};
                char path[100] = {0};
                int  tmpStart = -1;
                int  tmpEnd  = -1;
                char tmpName[80] = {0};
                
                sprintf(path,"/Application_Selector/%s/",sd_app_list->apps[sd_app_list->num_apps].app_name);
                sd_list(path,tmpBuf);
                
                int j=0;
                for(j=0;j<100;j++)
                {
                    if(tmpBuf[j] !='\0' && tmpStart == -1)  //start file
                    {
                        tmpStart = j;
                        tmpEnd = -1;
                    } 
                    else if(tmpBuf[j] == '\0' && tmpEnd == -1)//end file
                    {
                        tmpEnd = j;
                        memcpy(tmpName,tmpBuf+tmpStart,tmpEnd-tmpStart+1);
                        if(tmpName[0] != '.')
                        {
                            char *pdest = NULL;
                            pdest = strstr(tmpName,"_SW.bin");
                            if(pdest != NULL)
                            {
                                strcpy( sd_app_list->apps[sd_app_list->num_apps].sw_filename, tmpName );
                            }
                            
                            pdest = strstr(tmpName,"_HW.bin");
                            if(pdest != NULL)
                            {
                                strcpy( sd_app_list->apps[sd_app_list->num_apps].hw_filename, tmpName );
                            }
                        }
                        tmpStart = -1;
                    }
                }
                //iIndex ++;
                sd_app_list->num_apps++;
            }
            start = -1;
        }
    }
    if(sd_app_list->num_apps  == 0)
        return 2;
    return 0;
}

void Down_SW_BIN(app_list_struct* sd_app_list , alt_video_display* display)
{
    Display_Process(sd_app_list, display);
    //if there is not a _SW.bin ,then return
    if(strcmp(sd_app_list->apps[sd_app_list->target_app].sw_filename,"") == 0)
    {
        return ;
    }
    char szText[32]={0};
    int volumes_mounted;
    ////set spi clock divider coeff.
    sd_set_clock_to_max( 80000000 );
    usleep (1000);
    ////mount fat file system.
    volumes_mounted = sd_fat_mount_all();
    
    if( volumes_mounted <= 0 )
    {
        lcd_display( "\rSD Card Mount FAILED\n\n" );    
        return;
    }
    
    int i=0 , j=0;
    int fd =0;
    int ret_code =0;
    int file_length=0;
    int remaining_length = 0;  //remain file size
    int ret_length=0;
    int num_of_blocks =0;
    int block_size;
    int residual =0;
    int region_size;
    
    char fileName[255]={0};
    //sprintf(fileName,"/Application_Selector/%s/%s",app_list[iIndex].Dir,app_list[iIndex].sw_file_name);
    sprintf(fileName,"/Application_Selector/%s/%s",sd_app_list->apps[sd_app_list->target_app].app_name,sd_app_list->apps[sd_app_list->target_app].sw_filename);
    fd = sd_open(fileName,O_RDWR);
    file_length = sd_filesize(fd);
    remaining_length = file_length;
    char *buf = (char *)malloc(65536);
    
    ///epcs device open
    alt_flash_fd *my_epcs; //define handle
    flash_region* regions;
    int number_of_regions;
    
    my_epcs = alt_flash_open_dev(CFI_FLASH_NAME);//FLASHȡ
    if(my_epcs == NULL)
    {
        sprintf(szText,"\rOpen Flash  \nfailed\n");
        lcd_display(szText);
        
        sd_close(fd);
        
        return;
    }
    
    ret_code = alt_get_flash_info (my_epcs, &regions, &number_of_regions);//ȡоƬϢ
    int n = 0;
    
    for(i=0; i<number_of_regions; i++)
    {
        block_size = regions[i].block_size;
        region_size   = regions[i].region_size;
        
        if((file_length -regions[i].offset) >= region_size) //the current region is not enough
        {
            //write block
            ret_length  = sd_read(fd,buf,region_size);
            if (ret_length!=region_size)
            {
                printf("SD card read failed\n");
                alt_flash_close_dev(my_epcs);
                sd_close(fd);
                return ;
            }
            
            ret_code = alt_write_flash(my_epcs,regions[i].offset ,buf,region_size);
            if(ret_code != 0)
            {
                sprintf(szText,"\rwrite region%d \n block %d failed\n",i,n);
                lcd_display(szText);
                alt_flash_close_dev(my_epcs);
                sd_close(fd);
                return;
            }
            else
            {
                sd_app_list->apps[sd_app_list->target_app].progress += region_size;
                Display_Process(sd_app_list,display);
            
                int Down_File_Size = sd_app_list->apps[sd_app_list->target_app].progress;
                int Total_File_Size = sd_app_list->apps[sd_app_list->target_app].image_length;
                sprintf(szText,"\rdownload %d%%\n\n",Down_File_Size*100 /Total_File_Size );
                lcd_display(szText);
            }
        }
        else  //the current region is enough
        {
            num_of_blocks = (file_length-regions[i].offset) / block_size;
            residual = (file_length-regions[i].offset) % block_size;
            
            n = 0;
            for(j=0;j<num_of_blocks;j++)
            {   
                //write block
                ret_length  = sd_read(fd,buf,block_size);
                if (ret_length!=ret_length)
                {
                    printf("SD card read failed\n");
                    alt_flash_close_dev(my_epcs);
                    sd_close(fd);
                    return ;
                }
                
                ret_code = alt_write_flash(my_epcs,regions[i].offset + block_size*n,buf,block_size);
                if(ret_code != 0)
                {
                    sprintf(szText,"\rwrite region%d \nblock %d failed\n",i,n);
                    lcd_display(szText);
                    alt_flash_close_dev(my_epcs);
                    sd_close(fd);
                    return;
                }
                else
                {
                    sd_app_list->apps[sd_app_list->target_app].progress += block_size;
                    Display_Process(sd_app_list,display);
            
                    int Down_File_Size = sd_app_list->apps[sd_app_list->target_app].progress;
                    int Total_File_Size = sd_app_list->apps[sd_app_list->target_app].image_length;
                    sprintf(szText,"\rdownload %d%%\n\n",Down_File_Size*100 /Total_File_Size );
                    lcd_display(szText);
                }
                n++;
            }
            
            if (residual!=0)
            {               
                //write block
                ret_length  = sd_read(fd,buf,residual);
                if (ret_length!=residual)
                {
                    printf("SD card read failed\n");
                    alt_flash_close_dev(my_epcs);
                    sd_close(fd);
                    return;
                }
                
                ret_code = alt_write_flash(my_epcs, regions[i].offset+block_size*n,buf, residual); //дresidualֽ
                if (ret_code !=0)
                {
                    sprintf(szText,"\rwrite region%d \n block %d failed\n",i,n);
                    lcd_display(szText);
                    alt_flash_close_dev(my_epcs);
                    sd_close(fd);
                    return ;
                }
                else
                {
                    sd_app_list->apps[sd_app_list->target_app].progress += residual;
                    Display_Process(sd_app_list,display);
            
                    int Down_File_Size = sd_app_list->apps[sd_app_list->target_app].progress;
                    int Total_File_Size = sd_app_list->apps[sd_app_list->target_app].image_length;
                    sprintf(szText,"\rdownload %d%%\n\n",Down_File_Size*100 /Total_File_Size );
                    lcd_display(szText);
                }
            }
            break;  //to the end
        }
    }
    
    //close flash & sdcard
    alt_flash_close_dev(my_epcs);
    sd_close(fd);
    free(buf);
}

int Down_HW_BIN(app_list_struct* sd_app_list, alt_video_display* display)
{
    char szText[32]={0};
    int volumes_mounted;
    ////set spi clock divider coeff.
    sd_set_clock_to_max( 80000000 );
    usleep (1000);
    ////mount fat file system.
    volumes_mounted = sd_fat_mount_all();
    
    if( volumes_mounted <= 0 )
    {
        sprintf(szText,"\rSD Card Mount\nFAILED\n");
        lcd_display(szText);
        return -1;  
    }
    
    int i=0;
    int fd =0;
    int ret_code =0;
    int file_length=0;
    int remaining_length = 0;
    int ret_length=0;
    int num_block =0;
    int residual =0;
    char fileName[255] = {0};
    //sprintf(fileName,"/Application_Selector/%s/%s",app_list[iIndex].Dir,app_list[iIndex].hw_file_name);
    sprintf(fileName,"/Application_Selector/%s/%s",sd_app_list->apps[sd_app_list->target_app].app_name,sd_app_list->apps[sd_app_list->target_app].hw_filename);
    fd = sd_open(fileName,O_RDWR);
    file_length = sd_filesize(fd);
    remaining_length = file_length ;
    num_block = file_length/EPCS_BLOCK_SIZE;
    residual  = file_length%EPCS_BLOCK_SIZE;
    
    char *buf = (char *)malloc(EPCS_BLOCK_SIZE);
    
    
    ///epcs device open
    alt_flash_fd *my_epcs; //define handle
    flash_region* regions;
    int number_of_regions;
    
    my_epcs = alt_flash_open_dev(EPCS_FLASH_CONTROLLER_NAME);//FLASHȡ
    if(my_epcs == NULL)
    {
        sprintf(szText,"\rOpen EPCS  \nfailed\n");
        lcd_display(szText);
        
        sd_close(fd);
        return -1;
    }
    ret_code = alt_epcs_flash_get_info (my_epcs, &regions, &number_of_regions);//ȡоƬϢ
    
    //int n = 64;
    int n = 32;
    
    
    for (i=0;i<num_block;i++)
    {   
        ret_length  = sd_read(fd,buf,EPCS_BLOCK_SIZE);
        if (ret_length!=EPCS_BLOCK_SIZE)
        {
            alt_flash_close_dev(my_epcs);
            sd_close(fd);
            return -1;
        }
        ret_code = alt_epcs_flash_write(my_epcs, RECONFIG_ADDRESS + EPCS_BLOCK_SIZE*i,buf,EPCS_BLOCK_SIZE);
        if(ret_code != 0)
        {
            sprintf(szText,"\rwrite block %d\nfailed\n",n);
            lcd_display(szText);
            alt_flash_close_dev(my_epcs);
            sd_close(fd);
            return -1;
        }
        else
        {
            sd_app_list->apps[sd_app_list->target_app].progress += EPCS_BLOCK_SIZE;
            Display_Process(sd_app_list,display);
            
            int Down_File_Size = sd_app_list->apps[sd_app_list->target_app].progress;
            int Total_File_Size = sd_app_list->apps[sd_app_list->target_app].image_length;
            sprintf(szText,"\rdownload %d%%\n\n",Down_File_Size*100 /Total_File_Size );
            lcd_display(szText);
        }
        n++;
    }
    if (residual!=0)
    {   
        //write block 
        ret_length  = sd_read(fd,buf,residual);
        if (ret_length!=residual)
        {
            sprintf(szText,"\rsd card read %d\n failed\n",n);
            lcd_display(szText);
            sd_close(fd);
            return -1;
        }
        
        ret_code = alt_epcs_flash_write(my_epcs, RECONFIG_ADDRESS + EPCS_BLOCK_SIZE*num_block,buf, residual); //дresidualֽ
        if (ret_code !=0)
        {
            sprintf(szText,"\r write block %d\n failed\n",n);
            lcd_display(szText);
            
            alt_flash_close_dev(my_epcs);
            sd_close(fd);
            return -1;
        }
        else
        {
            sd_app_list->apps[sd_app_list->target_app].progress += residual;
            Display_Process(sd_app_list,display);
            
            sprintf(szText,"\rdownload 100%\n\n" );
            lcd_display(szText);
        }
    }
    sprintf(szText,"\rdownload \nComplete\n");
    lcd_display(szText);
    
    alt_flash_close_dev(my_epcs);
    sd_close(fd);
    free(buf);
}

void ComputeTotalSize(app_list_struct* sd_app_list)
{
    int volumes_mounted;
    volumes_mounted = sd_fat_mount_all();
    
    if( volumes_mounted <= 0 )
    {
        lcd_display( "\rSD Card Mount FAILED\n\n" );    
        return;
    }
    int fd = 0;
    int file_length = 0;
    char fileName[255]={0};
    
    if(strcmp(sd_app_list->apps[sd_app_list->target_app].sw_filename,"") != 0)
    {
        sprintf(fileName,"/Application_Selector/%s/%s",sd_app_list->apps[sd_app_list->target_app].app_name,sd_app_list->apps[sd_app_list->target_app].sw_filename);
        fd = sd_open(fileName,O_RDWR);
        file_length = sd_filesize(fd);
        sd_close(fd); 
    }
    
    sprintf(fileName,"/Application_Selector/%s/%s",sd_app_list->apps[sd_app_list->target_app].app_name,sd_app_list->apps[sd_app_list->target_app].hw_filename);
    fd = sd_open(fileName,O_RDWR);
    file_length = file_length + sd_filesize(fd);
    sd_close(fd);
    
    sd_app_list->apps[sd_app_list->target_app].image_length = file_length ;
}

void Display_Process(app_list_struct* sd_app_list , alt_video_display* display)
{
    char szText[128];
    int Total_File_Size = sd_app_list->apps[sd_app_list->target_app].image_length;
    int Down_File_Size  = sd_app_list->apps[sd_app_list->target_app].progress ;
    
    alt_video_display_clear_screen( display, 0x0 );
    
    //loading 
    sprintf(szText, "Loading......");
    vid_print_string_alpha(350, 300, COLOR_WHITE, 0xFFFFFFFF, tahomabold_20, display, szText);
    
    //progress
    vid_draw_round_corner_box ( 150, 350, 650, 380, 5, COLOR_B, DO_NOT_FILL, display);
    float percent = Down_File_Size*1.0/Total_File_Size;
    vid_draw_round_corner_box ( 150, 350, 150+percent*500, 380, 5, COLOR_B, DO_FILL, display);
    
    sprintf(szText, " %d%% ",Down_File_Size*100/Total_File_Size);
    vid_print_string_alpha(350, 352, COLOR_WHITE, 0xFFFFFFFF, tahomabold_20, display, szText);
    
    // update canvas
    alt_video_display_register_written_buffer( display );
}

int EPCS_Reconfig( int remote_update_base,int flash_base,int reconfig_offset,int watchdog_timeout,int width_of_flash )
{   
    int offset_shift;
    // Obtain upper 12 bits of 29-bit watchdog timeout value
    watchdog_timeout = watchdog_timeout >> 17;
    // Only enable the watchdog timer if its timeout value is greater than 0.
    if( watchdog_timeout > 0 )
    {
        // Set the watchdog timeout value
        IOWR( remote_update_base, 0x2, watchdog_timeout );
    }
    else
    {
        // Disable the watchdog timer
        IOWR( remote_update_base, 0x3, 0 );
    }
    
    // Calculate how much to shift the reconfig offset location:
    // width_of_flash == 8->offset_shift = 2.
    // width_of_flash == 16->offset_shift = 3
    offset_shift = (( width_of_flash / 8 ) + 1 );
    // Write the offset of the desired reconfiguration image in flash
    IOWR( remote_update_base, 0x4, reconfig_offset >> offset_shift );
    // Perform the reconfiguration by setting bit 0 in the
    // control/status register
    IOWR( remote_update_base, 0x20, 0x1 );
    return( 0 );
}

/*****************************************************************************
 *  Function: Check_IsExsist
 *
 *  Purpose:  Check demo whether has exsisted
 * 
 *  Returns: 1: yes   0: No
 ****************************************************************************/
int Check_IsExsist(app_list_struct* sd_app_list , int demo_address)
{
    int result = 0;
    char szText[32]={0};
    ///epcs device open
    alt_flash_fd *my_epcs; //define handle
    
    int ret_code = 0;
    
    
    my_epcs = alt_flash_open_dev(EPCS_FLASH_CONTROLLER_NAME);//Open flash and get handle
    if(my_epcs == NULL)
    {
        sprintf(szText,"\rOpen EPCS  \nfailed\n");
        lcd_display(szText);
        
        return result;
    }
   
    char buf[80] = {0};
    ret_code = alt_epcs_flash_read(my_epcs, demo_address, buf, 80);    
    
        if(strcmp(sd_app_list->apps[sd_app_list->target_app].app_name, buf) == 0)
        {
            result = 1;
        }
        else
        {
            ret_code = alt_epcs_flash_write(my_epcs, demo_address, sd_app_list->apps[sd_app_list->target_app].app_name, 80);
        }
    alt_flash_close_dev(my_epcs);
    return result;
}
